//
//  UltraDemoToast.m
//  Bloc
//
//  Created by Lin feihong on 10/27/12.
//  Copyright (c) 2012 Bloc. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>

#import "UltraDemoToast.h"

#define UltraDemoToastMagin 10.0
#define BlocScreenPortraitDistance 45.0
#define BlocScreenLandscapeDistance 40.0
#define UltraDemoToastMaxHeight 60.0
#define BlocSystemVersion [[[UIDevice currentDevice] systemVersion] floatValue]
#define Bloc_XCODE_5   0

@interface UltraDemoToast () {
    UILabel *_toastLabel;
    UIButton *_toastBtn;
    UltraDemoToastGravity _gravity;
    UltraDemoToastDuration _duration;
    
}

@property (nonatomic, retain) UIButton *toastBtn;

- (id)initWithText:(NSString *)text duration:(int)duration;
- (void)registerChangeStatusBarFrameNotification;
- (void)unregisterNotification;
- (void)statusBarOrientationWillChange:(NSNotification *)notification;
- (void)setOrientationLayoutSubviews:(UIInterfaceOrientation)orientation;
- (void)hide:(id)sender;

@end


@implementation UltraDemoToast

@synthesize duration = _duration;
@synthesize gravity = _gravity;
@synthesize toastBtn = _toastBtn;

#pragma mark - Initialize

+ (UltraDemoToast *)makeText:(NSString *)text duration:(int)duration{
    UltraDemoToast *toast = nil;
    if (text.length > 0)
    {
        toast = [[UltraDemoToast alloc] initWithText:text duration:duration];
    }
    return toast;
}

- (id)initWithText:(NSString *)text duration:(int)duration{
    if (self = [super init]) {
        _gravity = UltraDemoToastGravityBottom;
        _duration = duration;
        UIFont *font = [UIFont boldSystemFontOfSize:14.0];
        CGSize textSize = [text sizeWithFont:font constrainedToSize:CGSizeMake([[UIScreen mainScreen] bounds].size.width - BlocScreenLandscapeDistance, UltraDemoToastMaxHeight)];
        
        _toastLabel = [[UILabel alloc] initWithFrame:CGRectZero];
        _toastLabel.frame = CGRectMake(UltraDemoToastMagin / 2, UltraDemoToastMagin / 2, textSize.width + UltraDemoToastMagin, textSize.height + UltraDemoToastMagin);
        _toastLabel.text = text;
        _toastLabel.font = font;
        _toastLabel.textColor = [UIColor whiteColor];
        _toastLabel.textAlignment = NSTextAlignmentCenter;
        _toastLabel.numberOfLines = 0;
        _toastLabel.shadowColor = [UIColor darkGrayColor];
        _toastLabel.shadowOffset = CGSizeMake(1.0, 1.0);
        _toastLabel.backgroundColor = [UIColor clearColor];
        _toastLabel.clipsToBounds = YES;
        
        self.toastBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        self.toastBtn.frame = CGRectMake(0.0, 0.0, _toastLabel.frame.size.width + UltraDemoToastMagin, _toastLabel.frame.size.height + UltraDemoToastMagin);
        self.toastBtn.backgroundColor = [UIColor colorWithRed:0.000 green:0.000 blue:0.000 alpha:0.750];
        self.toastBtn.layer.masksToBounds = YES;
        self.toastBtn.layer.cornerRadius = 3.0;
        [self.toastBtn addTarget:self action:@selector(hide:) forControlEvents:UIControlEventTouchUpInside];
        self.toastBtn.autoresizingMask = UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin;
        [self.toastBtn addSubview:_toastLabel];
        
        [self registerChangeStatusBarFrameNotification];
        [self setOrientationLayoutSubviews:[[UIApplication sharedApplication] statusBarOrientation]];
    }
    return self;
}

- (void)dealloc {
    [self unregisterNotification];
}

#pragma mark - Register & unregister notification

- (void)registerChangeStatusBarFrameNotification {
    [[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(statusBarOrientationWillChange:)
												 name:UIApplicationWillChangeStatusBarOrientationNotification
											   object:nil];
}

- (void)unregisterNotification {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - According orientation change layout

- (void)statusBarOrientationWillChange:(NSNotification *)notification {
    if ([self.toastBtn superview]) {
        [self.toastBtn removeFromSuperview];
    }
    UIInterfaceOrientation orientation = [[notification.userInfo objectForKey:UIApplicationStatusBarOrientationUserInfoKey] intValue];
    [self setOrientationLayoutSubviews:orientation];
}

- (void)setOrientationLayoutSubviews:(UIInterfaceOrientation)orientation {
    CGPoint point = [UIApplication sharedApplication].keyWindow.center;
    self.toastBtn.center = CGPointMake(point.x, point.y);

    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 8.0) {
//        self.toastBtn.center = CGPointMake(point.x, BlocScreenPortraitDistance);
        [self performSelector:@selector(hide:) withObject:nil afterDelay:(_duration / 1000)];
        return;
    }
    
    switch (orientation) {
        case UIInterfaceOrientationPortrait:
        {
            self.toastBtn.transform = CGAffineTransformMakeRotation(0);
//            self.toastBtn.center = CGPointMake(point.x, BlocScreenPortraitDistance);
        }
            break;
        case UIInterfaceOrientationPortraitUpsideDown:
        {
            self.toastBtn.transform = CGAffineTransformMakeRotation(-M_PI);
//            self.toastBtn.center = CGPointMake(point.x,point.y*2- BlocScreenPortraitDistance);
        }
            break;
        case UIInterfaceOrientationLandscapeLeft:
        {
            self.toastBtn.transform = CGAffineTransformMakeRotation(-M_PI_2);
//            self.toastBtn.center = CGPointMake(BlocScreenPortraitDistance, point.y);
        }
            break;
        case UIInterfaceOrientationLandscapeRight:
        {
            self.toastBtn.transform = CGAffineTransformMakeRotation(M_PI_2);
//            self.toastBtn.center = CGPointMake(2*point.x - BlocScreenPortraitDistance, point.y);
        }
            break;
        default:
            break;
    }
    
    [self performSelector:@selector(hide:) withObject:nil afterDelay:(_duration / 1000)];
}

#pragma mark - Show toast

- (void)show:(BOOL)isSpecial {
    [[[UIApplication sharedApplication] keyWindow] addSubview:self.toastBtn];
    if (isSpecial) {
        [self setOrientationLayoutSubviews:UIInterfaceOrientationPortrait]; //toast固定显示方向 正屏
    }else {
        [self setOrientationLayoutSubviews:[[UIApplication sharedApplication] statusBarOrientation]];
    }
    
}

#pragma mark - Hide toast

- (void)hide:(id)sender {
    [self.toastBtn removeFromSuperview];
}

@end
