//
//  ViewController.m
//  UltraSDKDemo
//
//  Created by 魏太山 on 2021/7/27.
//

#import "ViewController.h"
#import <UltraSDK/UltraSDK.h>
#import <UltraBaseSDK/UIColor+HBExtension.h>
#import "UltraDemoToast.h"

@interface ViewController ()

/**日志数组*/
@property (nonatomic, strong) NSMutableArray *logArray;

/** 渠道选择 */
@property (nonatomic, strong) UIButton *channelButton;

/** 当前登录状态 */
@property (nonatomic, strong) UILabel *lbCurrentLoginStatus;

/** 日志信息 */
@property (nonatomic, strong) UITextView *logTextView;

/** 登录 */
@property (nonatomic, strong) UIButton *loginButton;

/** 支付 */
@property (nonatomic, strong) UIButton *payButton;

/** 角色注册*/
@property (nonatomic, strong) UIButton *roleRegisterButton;

/** 角色登录 */
@property (nonatomic, strong) UIButton *roleLoginButton;

/** 角色升级 */
@property (nonatomic, strong) UIButton *roleUpgradeButton;

/** 注销 */
@property (nonatomic, strong) UIButton *logoutButton;

/** 基础角色信息 */
@property (nonatomic, strong) UltraHDCBaseGameRoleInfo *baseRoleInfo;

/** 角色信息 */
@property (nonatomic, strong) UltraHDCGameRoleInfo *roleInfo;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    [self initViews];
            
    //监听用户登陆成功的通知 单机版游戏不用监听
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(loginNotification:)
                                                 name:GAME_PUBLIC_NOTIFICATION_NAME_LOGIN
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                                selector:@selector(iapPaymentFinished:)
                                                    name:GAME_PUBLIC_NOTIFICATION_NAME_IAPPURCHASE_FINISH
                                                  object:nil];
    
    //监听平台退出事件
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(platformExit:)
                                                name:GAME_PUBLIC_NOTIFICATION_NAME_PLATFORM_EXIT
                                              object:nil];
    
    //监听点击切换账号
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(changeAccountNotification:)
                                                 name:GAME_PUBLIC_NOTIFICATION_NAME_CHANGE_ACCOUNT
                                               object:nil];
    
    //添加通知CP踢出玩家下线通知(根据国家政策必须接入此代码)(必接)
    [[NSNotificationCenter defaultCenter] addObserver:self          selector:@selector(loginFaildNotification:) name:GAME_PUBLIC_NOTIFICATION_NAME_LOGONINVALID object:nil];
    
    //监听兑换码
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(giftCard:)
                                                 name:GAME_PUBLIC_NOTIFICATION_NAME_GIFT_CARD
                                               object:nil];
        
    // 退出登录
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(exitLoginNoti:)
                                                 name:GAME_PUBLIC_NOTIFICATION_NAME_EXIT_LOGIN
                                               object:nil];
    
    [self addObserver:self forKeyPath:@"logArray" options:NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld context:nil];

    [[self mutableArrayValueForKey:@"logArray"] addObject:@"UltraSDK参数："];
    [[self mutableArrayValueForKey:@"logArray"] addObject:@"渠道参数："];
    
    dispatch_time_t time=dispatch_time(DISPATCH_TIME_NOW, 3*NSEC_PER_SEC);
    dispatch_after(time, dispatch_get_main_queue(), ^{
        //模拟闪屏上报
        [[UltraPlatform sharedInstance] postSplashScreenEndSuccess:^(id  _Nonnull obj) {
            [[self mutableArrayValueForKey:@"logArray"] addObject:@"模拟闪屏上报成功"];

        } faild:^(id  _Nonnull obj) {
            [[self mutableArrayValueForKey:@"logArray"] addObject:@"模拟闪屏上报失败"];
        }];
    });
    
}

-(void)viewWillLayoutSubviews{
    
    NSInteger count = 3;
    CGFloat margin = 16.f;
    CGFloat buttonWidth = (self.view.frame.size.width - margin * (count + 1))  / count;
    CGFloat buttonHeight = 44.f;
    CGFloat height = self.view.frame.size.height - margin;
    
    
    self.channelButton.frame = CGRectMake((self.view.frame.size.width - 150) / 2.f , 20 + 4, 150, 30);
    
    self.lbCurrentLoginStatus.frame = CGRectMake(margin, CGRectGetMaxY(self.channelButton.frame), self.view.frame.size.width - margin * 2, 30);
    
    self.roleLoginButton.frame = CGRectMake(margin, height - buttonHeight, buttonWidth, buttonHeight);
    self.roleRegisterButton.frame = CGRectMake(CGRectGetMaxX(self.roleLoginButton.frame) + margin, height - buttonHeight, buttonWidth, buttonHeight);
    self.roleUpgradeButton.frame = CGRectMake(CGRectGetMaxX(self.roleRegisterButton.frame) + margin, height - buttonHeight, buttonWidth, buttonHeight);
    
    self.loginButton.frame = CGRectMake(margin, height - buttonHeight*2 - margin, buttonWidth, buttonHeight);
    self.payButton.frame = CGRectMake(CGRectGetMaxX(self.loginButton.frame) + margin, height - buttonHeight*2 - margin, buttonWidth, buttonHeight);
    self.logoutButton.frame = CGRectMake(CGRectGetMaxX(self.payButton.frame) + margin, height - buttonHeight*2 - margin, buttonWidth, buttonHeight);
    
    
    self.logTextView.frame = CGRectMake(margin, CGRectGetMaxY(self.lbCurrentLoginStatus.frame) + margin, (self.view.frame.size.width - margin * 2) ,CGRectGetMinY(self.loginButton.frame) - CGRectGetMaxY(self.lbCurrentLoginStatus.frame) - margin*2);
}

/** 初始化视图*/
- (void)initViews{
    
    self.logArray = @[].mutableCopy;
    self.view.backgroundColor = [UIColor hb_colorWithHexString:@"#EEEEEE"];
    self.channelButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.channelButton setTitleColor:[UIColor hb_colorWithHexString:@"#222222"] forState:UIControlStateNormal];
    [self.channelButton setTitle:@"当前渠道：" forState:UIControlStateNormal];
    [self.channelButton addTarget:self action:@selector(channelAction:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:self.channelButton];
    
    self.lbCurrentLoginStatus = [[UILabel alloc] init];
    self.lbCurrentLoginStatus.text = @"当前登录状态：未登录";
    [self.view addSubview:self.lbCurrentLoginStatus];
    
    self.logTextView = [[UITextView alloc] init];
    self.logTextView.layoutManager.allowsNonContiguousLayout = NO;
    self.logTextView.backgroundColor = [UIColor hb_colorWithHexString:@"#AAAAAA"];
    self.logTextView.layer.borderWidth = 1;
    self.logTextView.layer.borderColor = [UIColor blackColor].CGColor;
    [self.view addSubview:self.logTextView];
    
    //登录
    self.loginButton = [self getButtonWithTitle:@"登录" action:@selector(loginAction:) isBorder:NO];
    
    //支付
    self.payButton = [self getButtonWithTitle:@"支付" action:@selector(payAction:) isBorder:NO];

    //登录上报
    self.roleLoginButton = [self getButtonWithTitle:@"角色登录" action:@selector(roleLoginAction:) isBorder:NO];

    //注册上报
    self.roleRegisterButton = [self getButtonWithTitle:@"角色注册" action:@selector(roleRegisterAction:) isBorder:NO];

    //升级上报
    self.roleUpgradeButton = [self getButtonWithTitle:@"角色升级" action:@selector(roleupgradeAction:) isBorder:NO];

    //注销
    self.logoutButton = [self getButtonWithTitle:@"注销" action:@selector(logoutAction:) isBorder:NO];


}

- (void)initRoleData{
    //数据上报
    UltraHDCBaseGameRoleInfo *base = [UltraHDCBaseGameRoleInfo sharedInstance];
    base.channelUserId = @"111";
    base.gameUserId = @"2222";
    base.serverId = @"268";
    base.serverName = @"rreeww";
    base.roleId = @"268300111675";
    base.roleName = @"我叫QA";
    base.roleAvatar = @"https://dwz.cn/HiqovD5u";
    [[UltraPlatform sharedInstance] setBaseRoleInfoWithData:base];


    UltraHDCGameRoleInfo *roleInfo = [[UltraHDCGameRoleInfo alloc] init];
    roleInfo.level = @"1";
    roleInfo.vipLevel = @"2";
    roleInfo.levelExp = @"3";
    roleInfo.vipScore = @"4";
    roleInfo.rankLevel = @"5";
    roleInfo.rankExp = @"6";
    roleInfo.rankLeve2 = @"7";
    roleInfo.rankExp2 = @"8";
    roleInfo.gold1 = @"9";
    roleInfo.gold2 = @"10";
    roleInfo.cupCount1 = @"11";
    roleInfo.cupCount2 = @"12";
    roleInfo.maxCK = @"13";
    roleInfo.sumPay = @"14";
    roleInfo.teamId = @"15";
    roleInfo.teamName = @"16";
    roleInfo.itemComposes = @[@"17"];
    
    self.baseRoleInfo = base;
    self.roleInfo = roleInfo;
}

- (UIButton *)getButtonWithTitle:(NSString *)title action:(SEL)action isBorder:(BOOL)isBorder{
    UIButton* button = [UIButton buttonWithType:UIButtonTypeCustom];
    if (isBorder) {
        button.backgroundColor = [UIColor clearColor];
        button.layer.borderColor = [UIColor hb_colorWithHexString:@"#3F6DF8"].CGColor;
        button.layer.borderWidth = 1.f;
        [button setTitleColor:[UIColor hb_colorWithHexString:@"#3F6DF8"] forState:UIControlStateNormal];
    }else{
        button.backgroundColor = [UIColor hb_colorWithHexString:@"#3F6DF8"];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];

    }
    button.layer.cornerRadius = 22.f;
    button.layer.masksToBounds = YES;
    [button setTitle:title forState:UIControlStateNormal];
    button.titleLabel.font = [UIFont boldSystemFontOfSize:16];
    [button addTarget:self action:action forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:button];
    
    
    return button;
}


#pragma mark - 点击方法
- (void)channelAction:(UIButton *)button{

}

/**登录方法*/
- (void)loginAction:(UIButton *)button{
    [[UltraPlatform sharedInstance] enterLoginView];
}


/**支付方法*/
- (void)payAction:(UIButton *)button{
    if (![UltraPlatform sharedInstance].isLogined) {
        [[UltraDemoToast makeText:@"请先登录" duration:UltraDemoToastDurationNormal] show:NO];
        return;
    }
    
    GamePaymentParameters *parameterData = [[GamePaymentParameters alloc] init];
    parameterData.gamePropID = @"1000" ;
    parameterData.gameRole = @"HeroUGameRole";
    parameterData.cpOrder = [NSString stringWithFormat:@"%d",(arc4random() % 501) + 500];
    parameterData.callbackUrl = @"sdfasdfasdfasdfasd";
    [[UltraPlatform sharedInstance] iapPurchaseWithData:parameterData];
}

/**角色登录*/
- (void)roleLoginAction:(UIButton *)button{
    if (![UltraPlatform sharedInstance].isLogined) {
        [[UltraDemoToast makeText:@"请先登录" duration:UltraDemoToastDurationNormal] show:NO];
        return;
    }
    [self initRoleData];
    [[UltraPlatform sharedInstance] roleLoginWithGameRoleInfo:self.roleInfo];
}

/**角色升级*/
- (void)roleupgradeAction:(UIButton *)button{
    if (![UltraPlatform sharedInstance].isLogined) {
        [[UltraDemoToast makeText:@"请先登录" duration:UltraDemoToastDurationNormal] show:NO];
        return;
    }
    [self initRoleData];
    NSInteger level = [self.roleInfo.level integerValue];
    self.roleInfo.level = [NSString stringWithFormat:@"%ld",(long)++level];
    [[UltraPlatform sharedInstance] roleLevelUpWithGameRoleInfo:self.roleInfo];
}

/**角色注册*/
- (void)roleRegisterAction:(UIButton *)button{
    if (![UltraPlatform sharedInstance].isLogined) {
        [[UltraDemoToast makeText:@"请先登录" duration:UltraDemoToastDurationNormal] show:NO];
        return;
    }
    [self initRoleData];
    self.baseRoleInfo.roleId = [NSString stringWithFormat:@"%ld",arc4random() % 10000000000];
    self.baseRoleInfo.roleName = [@"我叫QA" stringByAppendingString:self.baseRoleInfo.roleId];
    [[UltraPlatform sharedInstance] roleRegisterWithGameRoleInfo:self.roleInfo];
}

/**注销*/
- (void)logoutAction:(UIButton *)button{
    
    if (![UltraPlatform sharedInstance].isLogined) {
        [[UltraDemoToast makeText:@"请先登录" duration:UltraDemoToastDurationNormal] show:NO];
        return;
    }
    self.lbCurrentLoginStatus.text = [NSString stringWithFormat:@"当前登录状态：已注销登录"];
    [[UltraPlatform sharedInstance] logout];
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary<NSKeyValueChangeKey,id> *)change context:(void *)context{
    [self setTextViewLog];
}

- (void)setTextViewLog{
    
    NSString* string = @"";
    for (id object in self.logArray) {

        NSString* str = @"";
        if ([object isKindOfClass:NSString.class]) {
            str = object;
        }else {
            str = [object hb_JSONString];
        }
        string = [string stringByAppendingString:str.length ? str : @""];
        string = [string stringByAppendingString:@"\n\n"];
    }
    
    self.logTextView.text = string;
    [self.logTextView scrollRangeToVisible:NSMakeRange(self.logTextView.text.length, 1)];

}
#pragma mark - 通知回调

/**登录回调*/
-(void)loginNotification:(NSNotification*)notification{
    
    NSDictionary* result = notification.object;
    BOOL isLoginSuccess = [result[GAME_PUBLIC_TAG_LOGIN_IS_SUCCESS] boolValue];
    [[self mutableArrayValueForKey:@"logArray"] addObject:result ? [NSString stringWithFormat:@"登录结果:\n %@",result] : @""];

    if (isLoginSuccess) {
        NSString * accessCode = result[GAME_PUBLIC_TAG_ACCESS_CODE];
        NSString * accessToken = result[GAME_PUBLIC_TAG_ULTRA_ACCESS_TOKEN];
        NSLog(@"用户登陆成功!");
        NSLog(@"==> AccessCode: %@", accessCode);
        NSLog(@"==> AccessToken: %@", accessToken);
        NSLog(@"==> sdkuserid: %@", [[UltraPlatform sharedInstance] getUserId]);
        NSLog(@"==> username: %@", [[UltraPlatform sharedInstance] getUserName]);
        NSString *message = [NSString stringWithFormat:@"登录成功 ----> CP处理 %@ ",[[UltraPlatform sharedInstance] getUserName]] ;
        [[UltraDemoToast makeText:message duration:3000.0f] show:NO];
        NSLog(@"--------------------------------------------------------> ") ;
        NSLog(@"<---------------------------------------------------------") ;
        self.lbCurrentLoginStatus.text = [NSString stringWithFormat:@"当前登录状态：已登录，cUid = %@",result[@"cUid"]];
        [self initRoleData];
        [[UltraPlatform sharedInstance] roleLoginWithGameRoleInfo:self.roleInfo];

    }
    else
    {
        NSLog(@"用户登陆失败!");
        int errorCode = [notification.object[GAME_PUBLIC_TAG_LOGIN_ERROR_CODE] intValue];
        if (errorCode == GAME_PUBLIC_CODE_LOGIN_ERROR_USER_NOT_EXIST)
        {
            NSLog(@"用户不存在");
        }
        else if (errorCode == GAME_PUBLIC_CODE_LOGIN_ERROR_WRONG_PASSWORD)
        {
            NSLog(@"密码错误");
        }
        else
        {
            NSLog(@"未知Login 错误 code");
        }
    }
}

/**支付回调*/
-(void)iapPaymentFinished:(NSNotification*)notification{
    
    
    BOOL isIapPurchaseSucess = [notification.object[GAME_PUBLIC_TAG_IAPPURCHASE_IS_SUCCESS] boolValue];
    [[self mutableArrayValueForKey:@"logArray"] addObject:notification.object ? [NSString stringWithFormat:@"支付结果%@:\n %@",isIapPurchaseSucess ? @"成功" : @"失败", notification.object] : @""];

    UltraPaymentOrder * paymentOrder = notification.object[GAME_PUBLIC_TAG_PAYMENT_ORDER];
    [[self mutableArrayValueForKey:@"logArray"] addObject:paymentOrder ? [NSString stringWithFormat:@"订单信息:\n %@",paymentOrder] : @""];
    if (isIapPurchaseSucess) {
        NSLog(@"iap购买成功");
        NSLog(@"===> 支付成功订单:");
        NSLog(@"===> iap订单id: %@", paymentOrder.orderId);
    } else {
        NSLog(@"iap购买失败");
        if(paymentOrder != nil){
            NSLog(@"===> 支付失败订单:");
               NSLog(@"===> iap订单id: %@", paymentOrder.orderId);
            NSLog(@"===> iap订单errorCode: %ld", (long)paymentOrder.errorCode);
            NSLog(@"===> iap订单errorDescription: %@", paymentOrder.errorDescription);
        }

    }
}

/**平台退出回调*/
-(void)platformExit:(NSNotification*)notification
{
    [[self mutableArrayValueForKey:@"logArray"] addObject:@"平台退出回调事件成功"];
    int exitCode = [notification.object[GAME_PUBLIC_TAG_PLATFORM_EXIT_CODE] intValue];
    if (exitCode == GAME_PUBLIC_CODE_PLATFORM_EXIT_PAYMENT)
    {
        NSLog(@"从支付模块退出!");
    }
}


/** 切换账号回调 */
- (void)changeAccountNotification:(NSNotification*)notification{
    [[self mutableArrayValueForKey:@"logArray"] addObject:@"切换账号成功，模拟退出通行证"];
    self.lbCurrentLoginStatus.text = [NSString stringWithFormat:@"当前登录状态：已注销登录"];
}

/** CP踢出玩家下线通知(根据国家政策必须接入此代码)*/
- (void)loginFaildNotification:(NSNotification *)noti{
    //获取踢下线的理由
    NSString *message = noti.userInfo[GAME_PUBLIC_CP_KICKOFF_REASON];
    if (message.length > 0) {
        //如果消息内容不为空,则需要CP制作弹框显示
       NSMutableDictionary *params = [[NSMutableDictionary alloc] init];
       //上报踢玩家下线结果，0-失败、1-成功
       params[GAME_PUBLIC_CP_KICKOFF_RESULT] = @"1" ;
       //必须上报给SDK,且直到CP上报成功过后，SDK才会停止消息通知,
       [[UltraPlatform sharedInstance] cpKickOffCallBackWithResult:params];

    } else {
        //这里是SDK服务器踢人下线通知、没有提示信息

    }
}

- (void)giftCard:(NSNotification *)noti{
    
    NSString* goodsId = noti.object[@"goodsId"];
    CGFloat amount = [noti.object[@"amount"] doubleValue];
    // 下面2行日志，仅供参考
    NSLog(@"goodsId = %@, amount = %lf", goodsId, amount);
    [[self mutableArrayValueForKey:@"logArray"] addObject:[NSString stringWithFormat:@"礼包码兑换中，goodsId = %@, amount = %lf", goodsId, amount]];
    GamePaymentParameters *parameterData = [[GamePaymentParameters alloc] init];
    parameterData.gamePropID = @"1000" ;
    parameterData.gameRole = @"gameRole" ;
    parameterData.cpOrder = [NSString stringWithFormat:@"%d",(arc4random() % 501) + 500];
    parameterData.callbackUrl = @"123456789";//内购回调地址，可由运营配置，运营配置了此参数
    [[UltraPlatform sharedInstance] redeemGiftCardOrCodeWithData:parameterData success:^(NSDictionary * _Nonnull dic) {
        // 下面是日志，仅供参考
        NSLog(@"礼包码兑换成功 = %@", dic);
        [[self mutableArrayValueForKey:@"logArray"] addObject:[NSString stringWithFormat:@"礼包码兑换成功"]];
    } failure:^(NSString * _Nonnull error) {
        // 下面是日志，仅供参考
        NSLog(@"礼包码兑换失败 = %@", error);
        [[self mutableArrayValueForKey:@"logArray"] addObject:[NSString stringWithFormat:@"礼包码兑换失败，%@", error]];
    }];
}

- (void)exitLoginNoti:(NSNotification *)notification{
    NSLog(@"-------> 退出登录成功") ;
}
@end
